//
//  AVPConfig.h
//
//  Created by shiping.csp on 2018/11/16.
//  Copyright © 2018年 com.alibaba.AliyunPlayer. All rights reserved.
//

#ifndef AVPConfig_h
#define AVPConfig_h

#include "AVPDef.h"
#include <corecrt_malloc.h>
#include <string.h>

namespace alivc_player {
    typedef struct _AVPConfig {
        /**
         @brief 直播最大延迟 默认5000毫秒
         */
        int maxDelayTime;

        /**
         @brief 卡顿后缓存数据的高水位，当播放器缓存数据大于此值时开始播放
         */
        int highBufferDuration;

        /**
         @brief 开始起播缓存区数据长度，默认500ms
         */
        int startBufferDuration;

        /**
         @brief 播放器最大的缓存数据长度，默认50秒
         */
        int maxBufferDuration;

        /**
         @brief 网络超时时间，默认15秒
         */
        int networkTimeout;

        /**
         @brief 网络重试次数，每次间隔networkTimeout，networkRetryCount=0则表示不重试，重试策略app决定，默认值为2
         */
        int networkRetryCount;

        /**
        @brief 当stop的时候是否保留最后一帧图像显示，默认显示
        */
        bool showLastFrameWhenStop;

        /**
        @brief 请求referer
        */
        char *referer;

        /**
        @brief httpProxy代理
        */
        char *httpProxy;

        /**
        @brief user Agent
        */
        char *userAgent;

        /**
        @brief 添加自定义header
        */
        char **httpHeaders;

        /**
        @brief 自定义header的数量
        */
        int headerCount;

        _AVPConfig()
        {
            referer = nullptr;
            httpProxy = nullptr;
            userAgent = nullptr;
            httpHeaders = nullptr;
            networkTimeout = 15000;
            networkRetryCount = 2;
            maxBufferDuration = 500000;
            startBufferDuration = 500;
            highBufferDuration = 3000;
            maxDelayTime = 5000;
            showLastFrameWhenStop = true;
            headerCount = 0;
        }

        _AVPConfig(const _AVPConfig &config)
        {
            referer = nullptr;
            httpProxy = nullptr;
            userAgent = nullptr;
            httpHeaders = nullptr;

            if (config.referer) {
                referer = strdup(config.referer);
            }
            if (config.httpProxy) {
                httpProxy = strdup(config.httpProxy);
            }
            if (config.userAgent) {
                userAgent = strdup(config.userAgent);
            }
            
            headerCount = config.headerCount;
            httpHeaders = new char *[headerCount];
            for (int i = 0; i < headerCount; i++) {
                httpHeaders[i] = strdup(config.httpHeaders[i]);
            }

            networkTimeout = config.networkTimeout;
            networkRetryCount = config.networkRetryCount;
            maxBufferDuration = config.maxBufferDuration;
            startBufferDuration = config.startBufferDuration;
            highBufferDuration = config.highBufferDuration;
            maxDelayTime = config.maxDelayTime;
            showLastFrameWhenStop = config.showLastFrameWhenStop;
        }

        _AVPConfig operator=(const _AVPConfig &config)
        {
            if (referer) {
                free(referer);
                referer = nullptr;
            }
            if (config.referer) {
                referer = strdup(config.referer);
            }
            if (httpProxy) {
                free(httpProxy);
                httpProxy = nullptr;
            }
            if (config.httpProxy) {
                httpProxy = strdup(config.httpProxy);
            }
            if (userAgent) {
                free(userAgent);
                userAgent = nullptr;
            }
            if (config.userAgent) {
                userAgent = strdup(config.userAgent);
            }

            if (headerCount > 0) {
                for (int i = 0; i < headerCount; i++) {
                    free(httpHeaders[i]);
                }
                delete[] httpHeaders;
                httpHeaders = nullptr;
            }
            headerCount = config.headerCount;
            httpHeaders = new char *[headerCount];
            for (int i = 0; i < headerCount; i++) {
                httpHeaders[i] = strdup(config.httpHeaders[i]);
            }

            networkTimeout = config.networkTimeout;
            networkRetryCount = config.networkRetryCount;
            maxBufferDuration = config.maxBufferDuration;
            startBufferDuration = config.startBufferDuration;
            highBufferDuration = config.highBufferDuration;
            maxDelayTime = config.maxDelayTime;
            showLastFrameWhenStop = config.showLastFrameWhenStop;

            return *this;
        }

        ~_AVPConfig()
        {
            if (referer) {
                free(referer);
            }
            if (httpProxy) {
                free(httpProxy);
            }
            if (userAgent) {
                free(userAgent);
            }
            if (headerCount > 0) {
                for (int i = 0; i < headerCount; i++) {
                    free(httpHeaders[i]);
                }
                delete[] httpHeaders;
                httpHeaders = nullptr;
            }
        }

    } AVPConfig;

    typedef struct _AVPCacheConfig {
        /**
        @brief 单个视频缓存的最大时长，单位秒，即某个视频的时长超过maxDuration将不会被缓存
        */
        /****
        @brief The maximum length of a single video that can be cached. Unit: seconds. Videos that exceed the maximum length are not cached.
        */
        int maxDuration;

        /**
        @brief 所有缓存最大占用空间，单位：MB
        */
        /****
        @brief The maximum cache memory size. Unit: MB.
        */
        int maxSizeMB;

        /**
        @brief 是否开启缓存。默认关闭。
        */
        /****
        @brief Enable or disable content caching. Default: disabled.
        */
        bool enable;

        /**
        @brief 缓存目录
        */
        /****
        @brief The cache directory.
        */
        char *path;

        _AVPCacheConfig()
        {
            maxDuration = 0;
            maxSizeMB = 0;
            enable = false;
            path = nullptr;
        }

        _AVPCacheConfig(const _AVPCacheConfig &config)
        {
            path = nullptr;
            if (config.path) {
                path = strdup(config.path);
            }

            maxDuration = config.maxDuration;
            maxSizeMB = config.maxSizeMB;
            enable = config.enable;
        }

        _AVPCacheConfig operator=(const _AVPCacheConfig &config)
        {
            if (path) {
                free(path);
                path = nullptr;
            }
            if (config.path) {
                path = strdup(config.path);
            }
            maxDuration = config.maxDuration;
            maxSizeMB = config.maxSizeMB;
            enable = config.enable;

            return *this;
        }

        ~_AVPCacheConfig()
        {
            if (path) {
                free(path);
                path = nullptr;
            }
        }
    } AVPCacheConfig;

    class ALIVC_EXTERN VidPlayerConfigGenerator {
    public:
        /**
        @brief 设置预览时间
        @param previewTime 预览时间，单位为秒
        */
        /****
        @brief Set the preview duration.
        @param previewTime The specified preview duration in seconds.
        */
        void setPreviewTime(int previewTime);

        /**
        @brief HLS标准加密设置UriToken
        @param MtsHlsUriToken 字符串
        */
        /****
        @brief Set a UriToken for HLS standard encryption.
        @param MtsHlsUriToken The UriToken.
        */
        void setHlsUriToken(const char *mtsHlsUriToken);

        /**
        @brief 生成playerConfig, 返回值使用后需要使用alivcFree释放。
        */
        /****
        @brief Generate playerConfig. the return value should be freed width alivcFree()
        */
        const char *generatePlayerConfig();

    private:
        void *mConfigData = nullptr;
    };


};// namespace alivc_player

#endif /* AVPConfig_h */
