﻿#ifndef AliPlayer_LISTENNER_H
#define AliPlayer_LISTENNER_H

#include <stdint.h>
#include <stdlib.h>

#include "AVPDef.h"
#include "AVPMediaInfo.h"

namespace alivc_player {

    class AliPlayer;

    class IAVPListener {

    public:
        /**
         @brief 播放器事件回调
         @param player 播放器player指针
         @param eventType 播放器事件类型，@see AVPEventType
         */
        virtual void onPlayerEvent(AliPlayer *player, AVPEventType eventType) = 0;

        /**
         @brief 播放器事件回调
         @param player 播放器player指针
         @param eventWithString 播放器事件类型
         @param description 播放器事件说明
        */
        virtual void onPlayerEvent(AliPlayer *player, AVPEventWithString eventWithString, const char *description) = 0;

        /**
        @brief 错误代理回调
        @param player 播放器player指针
        @param errorModel 播放器错误描述，参考AliVcPlayerErrorModel
        */
        virtual void onError(AliPlayer *player, AVPErrorModel *errorModel) = 0;

        /**
        @brief 视频大小变化回调
        @param player 播放器player指针
         @param width 视频宽度
         @param height 视频高度
         @param rotation 视频旋转角度
         */
        virtual void onVideoSizeChanged(AliPlayer *player, int width, int height, int rotation) = 0;

        /**
        @brief 视频当前播放位置回调
        @param player 播放器player指针
        @param position 视频当前播放位置
        */
        virtual void onCurrentPositionUpdate(AliPlayer *player, int64_t position) = 0;

        /**
         @brief 视频缓存位置回调
         @param player 播放器player指针
         @param position 视频当前缓存位置
         */
        virtual void onBufferedPositionUpdate(AliPlayer *player, int64_t position) = 0;

        /**
         @brief 视频缓冲进度回调
         @param player 播放器player指针
         @param progress 缓存进度0-100
         */
        virtual void onLoadingProgress(AliPlayer *player, float progress) = 0;

        /**
         @brief 获取track信息回调
         @param player 播放器player指针
         @param info track流信息数组 参考AVPTrackInfo
         */
        virtual void onTrackReady(AliPlayer *player, AVPTrackInfo *info[], int count) = 0;

        /**
         @brief track切换完成回调
         @param player 播放器player指针
         @param info 切换后的信息 参考AVPTrackInfo
         */
        virtual void onTrackChanged(AliPlayer *player, AVPTrackInfo *info) = 0;

        /**
        @brief 外挂字幕被添加
        @param player 播放器player指针
        @param trackIndex 字幕流索引
        @param URL 字幕url
        */
        /****
        @brief External subtitles have been added
        @param player Player pointer.
        @param trackIndex Subtitle stream index.
        @param URL subtitle url
        */
        virtual void onSubtitleExtAdded(AliPlayer *player, int64_t trackIndex, const char *URL) = 0;

        /**
        @brief 字幕显示回调
        @param player 播放器player指针
        @param trackIndex 字幕流索引
        @param subtitleId 字幕id
        @param subtitle 字幕显示的字符串
        */
        virtual void onSubtitleShow(AliPlayer *player, int64_t trackIndex, int64_t subtitleId, const char *subtitle) = 0;

        /**
        @brief 字幕隐藏回调
        @param player 播放器player指针
        @param trackIndex 字幕流索引
        @param subtitleId 字幕id
        */
        virtual void onSubtitleHide(AliPlayer *player, int64_t trackIndex, int64_t subtitleId) = 0;

        /**
         @brief 获取缩略图成功回调
         @param positionMs 指定的缩略图位置
         @param fromPos 此缩略图的开始位置
         @param toPos 此缩略图的结束位置
         @param image 缩图略图像指针,对于mac是NSImage，iOS平台是UIImage指针
         */
        virtual void onGetThumbnailSuc(int64_t positionMs, int64_t fromPos, int64_t toPos, void *image) = 0;

        /**
         @brief 获取缩略图失败回调
         @param positionMs 指定的缩略图位置
         */
        virtual void onGetThumbnailFailed(int64_t positionMs) = 0;

        /**
         @brief 播放器状态改变回调
         @param player 播放器player指针
         @param oldStatus 老的播放器状态 参考AVPStatus
         @param newStatus 新的播放器状态 参考AVPStatus
         */
        virtual void onPlayerStatusChanged(AliPlayer *player, AVPStatus oldStatus, AVPStatus newStatus) = 0;

        /**
        @brief 截图数据回调
        @param player 播放器player指针
        @param width 返回宽度
        @param height 返回高度
        @param pARGBBuffer 数据指针 ARGB
        */
        virtual void onSnapshotImageBuffer(AliPlayer *player, int width, int height, unsigned char *pARGBBuffer) = 0;
    };
};// namespace alivc_player

#endif// AliPlayer_LISTENNER_H
