//
//  AliMediaDownloader.h
//
//  Created by kongjian.kongjian on 2020/06/04.
//

#ifndef AliMediaDownloader_h
#define AliMediaDownloader_h

#include "AVPDef.h"
#include "AVPMediaInfo.h"
#include "AVPSource.h"
#include "AVDConfig.h"

namespace alivc_player {

    class AliMediaDownloader;

    class IAVDListener {
    public:
        /**
         @brief Download preparation completion callback.
         @param downloader Downloader pointer.
         @param mediaInfo See AVPMediaInfo
         */
        virtual void onPrepared(AliMediaDownloader *downloader, AVPMediaInfo *mediaInfo) = 0;

        /****
        @brief error callback.
        @param downloader Downloader pointer.
        @param errorModel Player error description. See AliVcPlayerErrorModel
        */
        virtual void onError(AliMediaDownloader *downloader, AVPErrorModel *errorModel) = 0;

        /****
        @brief Download progress callback.
        @param downloader Downloader pointer.
        @param percent Download progress: 0 to 100.
        */
        virtual void onDownloadingProgress(AliMediaDownloader *downloader, int percent) = 0;

        /****
        @brief Downloaded file processing progress callback.
        @param downloader Downloader pointer.
        @param percent Processing progress: 0 to 100.
        */
        virtual void onProcessingProgress(AliMediaDownloader *downloader, int percent) = 0;

        /****
        @brief Download completion callback.
        @param downloader Downloader pointer.
        */
        virtual void onCompletion(AliMediaDownloader *downloader) = 0;
    };

    class ALIVC_EXTERN AliMediaDownloader {
    public:
        static AliMediaDownloader *CreateMediaDownloader();

        virtual ~AliMediaDownloader() = default;

        /****
        @brief Delete a downloaded file.
        @param saveDir The path where the file is saved.
        @param vid     VID.
        @param format  Format.
        @param index   The download index corresponds to the VID.
        */
        static int deleteFile(const char *saveDir, const char *vid, const char *format, int index);

        /****
        @brief Use AVPVidStsSource to prepare for download.
        @param source vid STS information.
        */
        virtual void prepareWithVid(AVPVidStsSource *source) = 0;

        /****
        @brief Update the AVPVidStsSource information when the authentication expires.
        @param source vid STS information.
        */
        virtual void updateWithVid(AVPVidStsSource *source) = 0;

        /****
        @brief Use AVPVidAuthSource to prepare for download.
        @param source vid Authentication information.
        */
        virtual void prepareWithPlayAuth(AVPVidAuthSource *source) = 0;

        /****
        @brief Update the AVPVidAuthSource information when the authentication expires.
        @param source vid Authentication information.
        */
        virtual void updateWithPlayAuth(AVPVidAuthSource *source) = 0;

        /****
        @brief Set a path to save the downloaded file.
        @param dir The specified path.
        */
        virtual void setSaveDirectory(const char *dir) = 0;

        /****
        @brief Start downloading.
        */
        virtual void start() = 0;

        /****
        @brief Pause downloading.
        */
        virtual void stop() = 0;

        /****
        @brief Delete the downloaded file.
        */
        virtual void deleteDownloadedFile() = 0;

        /****
        @brief Query download config. do not free the return value
        */
        virtual AVDConfig *getConfig() = 0;

        /****
        @brief Modify download config.
        */
        virtual void setConfig(AVDConfig *config) = 0;

        /****
        @brief Specify the indexes of the tracks to be downloaded.
        @param trackIndex Use the prepare callback to retrieve all track indexes.
        */
        virtual void selectTrack(int trackIndex) = 0;

        /****
        @brief Set a URL conversion callback. URL conversion is typically used in P2P broadcasting.
        @param callback The function pointer of the callback.
        */
        virtual void setPlayUrlConvertCallback(PlayURLConvertCallback callback) = 0;

        /**
        @brief set callback listener
        */
        virtual void setListener(IAVDListener *listener) = 0;

        /**
        @brief get callback listener
        */
        virtual IAVDListener* getListener() = 0;

        /****
        @brief Query the path of the downloaded file. do not need to free
        */
        virtual const char *getDownloadedFilePath() = 0;
    };

};// namespace alivc_player

#endif /* AliMediaDownloader_h */
