#ifndef APSARAVIDEO_PLAYER_H
#define APSARAVIDEO_PLAYER_H

#include <stdint.h>
#include <stdlib.h>
#include <windows.h>

#include "AVPConfig.h"
#include "AVPDef.h"
#include "AVPListener.h"
#include "AVPMediaInfo.h"
#include "AVPSource.h"

namespace alivc_player {

    typedef void (*ApsaraPlayerLogCallback)(void *userData, int prio, char *buf);

    void ALIVC_EXTERN InitPrivateService(const char *fileContentBuffer, int bufferLength);
    void ALIVC_EXTERN InitPrivateService(const char *filePath);

    class ALIVC_EXTERN AliPlayer {
    public:
        static AliPlayer *CreatePlayer();

        virtual ~AliPlayer() = default;
        /**
        @brief 使用url方式来播放视频
        @see AVPUrlSource
        */
        virtual void setSource(const AVPUrlSource &source) = 0;

        /**
        @brief 用vid和sts来播放视频，临时AccessKeyId、AccessKeySecret和SecurityToken：开启RAM授权，并通过STS授权系统提供的OpenAPI或SDK获取的AccessKeyId、AccessKeySecret和SecurityToken，用于播放和下载请求参数明细：https://help.aliyun.com/document_detail/28788.html
        @see AVPVidStsSource
        */
        virtual void setSource(const AVPVidStsSource &source) = 0;

        /**
        @brief 用vid和MPS信息来播放视频，视频转码服务用户使用播放方式。部分参数参考:https://help.aliyun.com/document_detail/53522.html
        @see AVPVidMpsSource
        */
        virtual void setSource(const AVPVidMpsSource &source) = 0;

        /**
        @brief 使用vid+playauth方式播放。playauth获取方法：https://help.aliyun.com/document_detail/57294.html
        @see AVPVidAuthSource
        */
        virtual void setSource(const AVPVidAuthSource &source) = 0;

        /**
        @brief 播放准备，异步
        */
        virtual void prepare() = 0;

        /**
        @brief 开始播放
        */
        virtual void start() = 0;

        /**
        @brief 暂停播放
        */
        virtual void pause() = 0;

        /**
        @brief 重置播放
        */
        virtual void reset() = 0;

        /**
        @brief 停止播放
        */
        virtual void stop() = 0;

        /**
        @brief 销毁播放器
        */
        virtual void destroy() = 0;

        /**
        @brief 暂停的时候重新绘制
        */
        virtual void redraw() = 0;

        /**
        @brief 截图
        */
        virtual void snapshot() = 0;

        /**
        @brief 设置监听
        */
        virtual void setListener(IAVPListener *listener) = 0;

        /**
        @brief 获取监听
        */
        virtual IAVPListener *getListener() = 0;

        /**
        @brief 获取主时钟pts
        */
        virtual int64_t getMasterClockPts() = 0;

        /**
        @brief 跳转到指定的播放位置
        @param time 新的播放位置, 单位ms
        @param seekMode seek模式，参考AVPSeekMode
        */
        virtual void seekToTime(int64_t time, AVPSeekMode seekMode) = 0;

        /**
        @brief 根据trackIndex，切换清晰度
        @param trackIndex 选择清晰度的index，SELECT_AVPTRACK_TYPE_VIDEO_AUTO代表自适应码率
        */
        virtual void selectTrack(int trackIndex) = 0;

        /**
        @brief 获取媒体信息，包括track信息
        */
        virtual AVPMediaInfo getMediaInfo() = 0;

        /**
        @brief 获取当前播放track
        @param type track类型
        */
        virtual AVPTrackInfo getCurrentTrack(AVPTrackType type) = 0;

        /**
        @brief 获取指定位置的缩略图
        @param positionMs 代表在哪个指定位置的缩略图
        */
        virtual void getThumbnail(int64_t positionMs) = 0;

        /**
        @brief 用于跟踪debug信息
        */
        virtual void setTraceID(const char *traceID) = 0;

        /**
        @brief 播放器设置
        */
        virtual void setConfig(AVPConfig *config) = 0;

        /**
        @brief 获取播放器设置
        */
        virtual AVPConfig *getConfig() = 0;

        /**
        @brief 设置缓存配置
        @param AVPCacheConfig 缓存配置。{@link AVPCacheConfig}。
        @return 设置成功返回YES
        */
        /****
        @brief Modify cache configuration.
        @param AVPCacheConfig Cache configuration. {@link AVPCacheConfig}.
        @return If the cache configuration is modified, YES is returned.
        */
        virtual bool setCacheConfig(AVPCacheConfig *cacheConfig) = 0;

        /**
        @brief 根据url获取缓存的文件名。
        @brief 如果没有实现delegate，将会采用默认逻辑生成文件名。
        @param URL URL
        @return 最终缓存的文件名（不包括后缀。缓存文件的后缀名统一为.alv）。使用后需要使用alivcFree释放。
        */
        /****
        @brief Query the name of the cached file with the specified URL. 
        @brief If the delegation failed, the default logic is used to generate the file name.
        @param URL The URL of the cached file.
        @return The name of the cached file. The file extension is not included. All files use the .alv extension. should be freed width alivcFree()
        */
        virtual const char *getCacheFilePath(const char *URL) = 0;


        /**
        @brief 根据vid获取缓存的文件名。如果有自定义的规则。
        @param vid        视频id
        @param format     视频格式
        @param definition 视频清晰度
        @param previewTime 试看时长, 0则忽略该参数
        @return 最终缓存的文件名（不包括后缀。缓存文件的后缀名统一为.alv）。使用后需要使用alivcFree释放。
        */
        /****
        @brief Query the name of the cached file with the specified URL. 
        @param vid        The VID of the video.
        @param format     The format of the video.
        @param definition The definition of the video.
        @param previewTime The preview duration of the video. if 0, ignore this param
        @return The name of the cached file. The file extension is not included. All files use the .alv extension.should be freed width alivcFree()
        */
        virtual const char *getCacheFilePath(const char *vid, const char *format, const char *definition, int previewTime) = 0;

        /**
        @brief 释放内部分配的内存
        @see getCacheFilePath
        */
        static void alivcFree(void *buffer);

        /**
        @brief 添加外挂字幕。
        @param URL 字幕地址
        */
        /****
        @brief Add external subtitles
        @param URL subtitle address
        */
        virtual void addExtSubtitle(const char *URL) = 0;

        /**
        @brief 选择外挂字幕
        @param trackIndex 字幕索引
        @param enable true：选择，false：关闭
        */
        /****
        @brief Select external subtitles
        @param trackIndex caption index
        @param enable true: select, false: close
        */
        virtual void selectExtSubtitle(int trackIndex, bool enable) = 0;

        /**
        @brief 获取SDK版本号信息
        */
        static const char *getSDKVersion();

        /**
        @brief 设置是否静音
        */
        virtual void setMute(bool muted) = 0;

        /**
        @brief 获取是否静音
        */
        virtual bool isMuted() = 0;

        /**
        @brief 设置播放速率，0.5-2.0之间，1为正常播放
        */
        virtual void setRate(float rate) = 0;

        /**
        @brief 获取播放速率，0.5-2.0之间，1为正常播放
        */
        virtual float getRate() = 0;

        /**
        @brief 设置是否开启硬件解码
        */
        virtual void enableHardwareDecoder(bool bEnabled) = 0;

        /**
        @brief 获取是否开启硬件解码
        */
        virtual bool isEnabledHardwareDecoder() = 0;

        /**
        @brief 设置是否循环播放
        */
        virtual void setLoop(bool bLoop) = 0;

        /**
        @brief 获取是否循环播放
        */
        virtual bool isLoop() = 0;

        /**
        @brief 设置是否自动播放
        */
        virtual void setAutoPlay(bool bAutoPlay) = 0;

        /**
        @brief 获取是否自动播放
        */
        virtual bool isAutoPlay() = 0;

        /**
        @brief 设置渲染镜像模式
        @see AVPMirrorMode
        */
        virtual void setMirrorMode(AVPMirrorMode mirrorMode) = 0;

        /**
        @brief 获取渲染镜像模式
        @see AVPMirrorMode
        */
        virtual AVPMirrorMode getMirrorMode() = 0;

        /**
        @brief 设置渲染旋转模式
        @see AVPRotateMode
        */
        virtual void setRotateMode(AVPRotateMode rotateMode) = 0;

        /**
        @brief 获取渲染旋转模式
        @see AVPRotateMode
        */
        virtual AVPRotateMode getRotateMode() = 0;

        /**
        @brief 设置渲染填充模式
        @see AVPScalingMode
        */
        virtual void setScalingMode(AVPScalingMode scalingMode) = 0;

        /**
        @brief 获取渲染填充模式
        @see AVPScalingMode
        */
        virtual AVPScalingMode getScalingMode() = 0;

        /**
        @brief 设置播放器的视图view
        */
        virtual void setView(void *view) = 0;

        /**
        @brief 获取播放器的视图view
        */
        virtual void *getView() = 0;

        /**
        @brief 获取视频的宽度
        */
        virtual int getWidth() = 0;

        /**
        @brief 获取视频的高度
        */
        virtual int getHeight() = 0;

        /**
        @brief 获取视频的旋转角度，从metadata中获取出来
        */
        virtual int getRotation() = 0;

        /**
        @brief 设置播放器的音量
        */
        virtual void setVolume(float volume) = 0;

        /**
        @brief 获取播放器的音量
        */
        virtual float getVolume() = 0;

        /**
        @brief 获取视频的长度, 单位ms
        */
        virtual int64_t getDuration() = 0;

        /**
        @brief 获取当前播放位置
        */
        virtual int64_t getCurrentPosition() = 0;

        /**
        @brief 获取已经缓存的位置
        */
        virtual int64_t getBufferedPosition() = 0;

        /**
        @brief 设置是否打开log输出
        */
        static void enableLog(AVPLogLevel level);

        /*
        * set log callback
        */
        static void setLogCallback(ApsaraPlayerLogCallback func, void *arg);
    };
};    // namespace alivc_player
#endif// APSARAVIDEO_PLAYER_H
